#!/bin/bash
#

# buildPython.sh <gcc47|gcc473> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <gcc47|gcc473> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

make_tarball()
{
    echo ""
    set -x
    cd ${PRODUCTS}
    tarname=${package}-${pkgdotver}-${OS}-${plat}-${qualdir}.tar.bz2
    tar cjf ${tarname} \
       ${package}/${pkgver}.version/${flvr}_${qualver} \
       ${package}/${pkgver}/ups \
       ${extras} \
       ${package}/${pkgver}/${flvr}-${qualdir}
    cd ${thisdir}
    set +x
    exit 0
}

make_source_tarball()
{
    set -x
    cd ${PRODUCTS}
    tar cjf ${package}-${pkgdotver}-source.tar.bz2 \
       ${package}/${pkgver}/*.sh \
       ${package}/${pkgver}/ups \
       ${source_extras} \
       ${package}/${pkgver}/source
    cd ${thisdir}
    set +x
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`
    qualver=`echo ${basequal} | sed -e 's/:/_/g'`

}

check_with_lsb_release ()
{
   plat=`uname -p`
   if [ `lsb_release -d | cut -f2 | cut  -f1 -d" "` = "Ubuntu" ]
   then
      OS1=ub
      OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
      OS=${OS1}${OSnum}
      plat=`uname -m`
   elif [ ${OS1} = "Linux" ]
   then
      OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
      OS=${OS1}${OSnum}
      # Scientific Linux - slf should work
      if [ `lsb_release -d | cut  -f3 -d" "` = "SL" ]
      then
	 OS=slf${OSnum}
      # Scientific Linux Fermi
      elif [ `lsb_release -d | cut  -f3 -d" "` = "SLF" ]
      then
	 OS=slf${OSnum}
      #
      elif [ `lsb_release -i | cut -f2` = "ScientificFermi" ]
      then
	 OS=slf${OSnum}
      # pretend that SL6 is the same as SLF6
      elif [ `lsb_release -i | cut -f2` = "Scientific" ]
      then
	 OS=slf${OSnum}
      # pretend that CentOS is SLF
      elif [ `lsb_release -i | cut -f2` = "CentOS" ]
      then
	 OS=slf${OSnum}
      # pretend that RedHatEnterpriseServer is SLF
      elif [ `lsb_release -i | cut -f2` = "RedHatEnterpriseServer" ]
      then
	 OS=slf${OSnum}
      # Scientific Linux CERN
      elif [ `lsb_release -d | cut  -f4 -d" "` = "SLC" ]
      then
	 OS=slc${OSnum}
      elif [ `lsb_release -d | cut  -f4 -d" "` = "LTS" ]
      then
	 OS=slf${OSnum}
      # unrecognized - pretend that this is SLF
      else
	 OS=slf${OSnum}
      fi
   fi
}

default_names ()
{
      OSnum1=`uname -r | cut -f1 -d"."`
      OSnum2=`uname -r | cut -f2 -d"."`
      OS=${OS1}${OSnum1}${OSnum2}
      plat=`uname -p`
}

check_linux ()
{
   plat=`uname -p`
   if [ -e /etc/system-release-cpe ]
   then
      OSnum=`cat /etc/system-release-cpe | cut -f5 -d":" |  cut -f1 -d"."`
      if [ `cat /etc/redhat-release | cut  -f4 -d" "` = "SLC" ]
      then
         OS=slc${OSnum}
      else
         OS=slf${OSnum}
      fi
   elif [ -e /etc/redhat-release ]
   then
      if [ `cat /etc/redhat-release | cut  -f4 -d" "` = "SLC" ]
      then
         OS=slc${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f6 -d" " |  cut -f1 -d"."`
      elif [ `cat /etc/redhat-release | cut  -f3 -d" "` = "Fermi" ]
      then
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      elif [ `cat /etc/redhat-release | cut  -f3 -d" "` = "SLF" ]
      then
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      else
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      fi
   else
      default_names
   fi
}

get_platform_name()
{
    source ${PRODUCTS}/setup

    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
	plat=`uname -m`
	OSnum=`uname -r | cut -f1 -d"."`
	#OS=${OS1}${OSnum}
	mactype=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | wc -w`
	if [ ${mactype} -gt 0 ]
	then
	   macos=`system_profiler  -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1,2`
	   macos1=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1`
	   macos2=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 2`
	else 
	   macos=`system_profiler  -xml -detailLevel mini SPSoftwareDataType | grep "OS X" | cut -d ' ' -f 3 | cut -d . -f 1,2`
	   macos1=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep "OS X" | cut -d ' ' -f 3 | cut -d . -f 1`
	   macos2=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep "OS X" | cut -d ' ' -f 3 | cut -d . -f 2`
	fi
	OS=osx${macos1}${macos2}
    elif [ "${flvr0}" = "Linuxppc" ]
    then
       OS=slf5
       plat=`uname -p`
    elif [ -e /usr/bin/lsb_release ]
    then
       check_with_lsb_release
    elif [ ${OS1} = "Linux" ]
    then
       check_linux
    else
       default_names
    fi

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${PRODUCTS}/${package}/${pkgver}/${flvr}-${qualdir}
    sourcedir=${PRODUCTS}/${package}/${pkgver}/source
}

select_gcc()
{
    if [ "${basequal}" = "e2" ] ||  [ "${basequal}" = "gcc47" ]
    then
       gccdotver=4.7
       gccver=v4_7_1
    elif [ "${basequal}" = "e3" ] ||  [ "${basequal}" = "gcc473" ]
    then
       gccdotver=4.7
       gccver=v4_7_3
    elif [ "${basequal}" = "gcc46" ]
    then
       gccdotver=4.6
       gccver=v4_6_1
    fi
}

setup_gcc()
{
    setup gcc ${gccver}
    gccshortver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
    gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
    if [ ${gccshortver} = ${gccdotver} ]
    then
      echo " OK - gcc version is ${gccfullver}"
    else
      echo "ERROR: wrong gcc version ${gccfullver}"
      exit 1
    fi
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

basequal=${1}
maketar=${2}
sourcetar=${3}

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=python
pkgver=v2_7_3
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
package_uc=`echo ${package} | tr '[:lower:]' '[:upper:]'`

get_this_dir

if [ "${sourcetar}" = "source" ]
then
  make_source_tarball
fi

set_qualifiers

get_platform_name

select_gcc

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_tarball
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

setup_gcc

mkdir -p ${pkgdir}/build
if [ ! -d ${pkgdir}/build ]
then
   echo "ERROR: failed to create ${pkgdir}/build"
   echo "       Something is very wrong"
   exit 1
fi

set -x

cd ${pkgdir}/build
${thisdir}/source/Python-${pkgdotver}/configure --prefix=${pkgdir} --enable-shared
make
make install

set +x

# replace idle, pydoc, and smtpd.py with the scripts which use #!/usr/bin/env python
if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi
cd ${pkgdir}/bin
cp -p ${thisdir}/source/Python-${pkgdotver}/Tools/scripts/2to3 .
cp -p ${thisdir}/source/Python-${pkgdotver}/Tools/scripts/pydoc .
cp -p ${thisdir}/source/Python-${pkgdotver}/Tools/scripts/idle .
cp -p ${thisdir}/source/Python-${pkgdotver}/Lib/smtpd.py .
mv python2.7-config python2.7-config.bak
cat python2.7-config.bak | sed 's%^#!.*python2.7$%#!/usr/bin/env python%' > python2.7-config
chmod +x python2.7-config
cd ${pkgdir}/lib/pkgconfig
mv python-2.7.pc python-2.7.pc.bak
cat python-2.7.pc.bak | sed 's%^prefix=.*$%prefix=${PYTHONHOME}%' > python-2.7.pc

declare_product

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual}
echo "${package} is installed at ${PYTHONHOME}"
echo ""
echo "IMPORTANT: you also need to edit ${PYTHONHOME}/lib/python2.7/config/Makefile"
echo ""

# this must be last because it will exit
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_tarball
fi

exit 0
